namespace Eco.Mods.TechTree
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using Eco.Gameplay.Blocks;
    using Eco.Gameplay.Components;
    using Eco.Gameplay.Components.Auth;
    using Eco.Gameplay.DynamicValues;
    using Eco.Gameplay.Economy;
    using Eco.Gameplay.Housing;
    using Eco.Gameplay.Interactions;
    using Eco.Gameplay.Items;
    using Eco.Gameplay.Minimap;
    using Eco.Gameplay.Objects;
    using Eco.Gameplay.Players;
    using Eco.Gameplay.Property;
    using Eco.Gameplay.Skills;
    using Eco.Gameplay.Systems.TextLinks;
    using Eco.Gameplay.Pipes.LiquidComponents;
    using Eco.Gameplay.Pipes.Gases;
    using Eco.Gameplay.Systems.Tooltip;
    using Eco.Shared;
    using Eco.Shared.Math;
    using Eco.Shared.Localization;
    using Eco.Shared.Serialization;
    using Eco.Shared.Utils;
    using Eco.Shared.View;
    using Eco.Shared.Items;
    using Eco.Gameplay.Pipes;
    using Eco.World.Blocks;

    [Serialized]
    [RequireComponent(typeof(OnOffComponent))]
    [RequireComponent(typeof(PropertyAuthComponent))]
    [RequireComponent(typeof(MinimapComponent))]
    [RequireComponent(typeof(LinkComponent))]
    [RequireComponent(typeof(FuelSupplyComponent))]
    [RequireComponent(typeof(FuelConsumptionComponent))]
    [RequireComponent(typeof(PowerGridComponent))]
    [RequireComponent(typeof(PowerGeneratorComponent))]
    [RequireComponent(typeof(SolidGroundComponent))]
    public partial class NuclearReactorObject :
        WorldObject,
        IRepresentsItem
    {
        public override LocString DisplayName { get { return Localizer.DoStr("Nuclear Reactor"); } }

        public virtual Type RepresentedItemType { get { return typeof(NuclearReactorItem); } }

        private static Type[] fuelTypeList = new Type[]
       {
            typeof(NuclearFuelCellItem),
       };

        protected override void Initialize()
        {
            this.GetComponent<MinimapComponent>().Initialize(Localizer.DoStr("Power"));
            this.GetComponent<FuelSupplyComponent>().Initialize(2, fuelTypeList);
            this.GetComponent<FuelConsumptionComponent>().Initialize(10000);
            this.GetComponent<PowerGridComponent>().Initialize(10, new ElectricPower());
            this.GetComponent<PowerGeneratorComponent>().Initialize(25000);
        }

        public override void Destroy()
        {
            base.Destroy();
        }

    }

    [Serialized]
    public partial class NuclearReactorItem :
        WorldObjectItem<NuclearReactorObject>
    {
        public override LocString DisplayName { get { return Localizer.DoStr("Nuclear Reactor"); } }
        public override LocString DisplayDescription { get { return Localizer.DoStr("This advanced piece of machinery generates vast amounts of power using the power of nuclear fission"); } }

        static NuclearReactorItem()
        {

        }

        [Tooltip(7)] private LocString PowerConsumptionTooltip { get { return new LocString(string.Format(Localizer.DoStr("Consumes: {0}w from fuel"), Text.Info(10000))); } }
        [Tooltip(8)] private LocString PowerProductionTooltip { get { return new LocString(string.Format(Localizer.DoStr("Produces: {0}w"), Text.Info(25000))); } }
    }

    [Serialized]
    [RequiresModule(typeof(ComputerLabObject))]
    [RequiresSkill(typeof(ElectronicsSkill), 3)]//skill used, level required
    public class NuclearReactorItemRecipe : Recipe //unique name of the recipe
    {
        public NuclearReactorItemRecipe()
        {
            this.Products = new CraftingElement[]
            {
                new CraftingElement<NuclearReactorItem>(1), //what is crafted, ammount, can craft multible products at a time
            };

            this.Ingredients = new CraftingElement[] //can't be empty or zero, can use multible ingredients at a time
            {
                new CraftingElement<ConcreteItem>(typeof(IndustrySkill), 1000f, IndustrySkill.MultiplicativeStrategy),
                new CraftingElement<SteelPlateItem>(typeof(IndustrySkill), 500f, IndustrySkill.MultiplicativeStrategy),
                new CraftingElement<CircuitItem>(typeof(ElectronicsSkill), 100f, ElectronicsSkill.MultiplicativeStrategy),
                new CraftingElement<ServoItem>(typeof(ElectronicsSkill), 50f, ElectronicsSkill.MultiplicativeStrategy),
                new CraftingElement<HeatSinkItem>(typeof(IndustrySkill), 50f, IndustrySkill.MultiplicativeStrategy),
            };

            this.CraftMinutes = CreateCraftTimeValue(typeof(NuclearReactorItemRecipe), Item.Get<NuclearReactorItem>().UILink(), 120f, typeof(IndustrySkill)); //from arrows recipe, 2 sec at max lvl

            this.Initialize(Localizer.DoStr("Nuclear Reactor"), typeof(NuclearReactorItemRecipe)); //visible name at the crafting station

            CraftingComponent.AddRecipe(typeof(RoboticAssemblyLineObject), this); //where is it crafted
        }
    }

}